@@ -6,12 +6,16 @@
 /* SYNOPSIS
 /*	#include "bounce_service.h"
 /*
-/*	int     bounce_notify_verp(flags, service, queue_name, queue_id, sender,
+/*	int     bounce_notify_verp(flags, service, queue_name, queue_id,
+/*					encoding, sendopts, sender,
 /*					dsn_envid, dsn_ret, verp_delims,
 /*					templates)
 /*	int	flags;
+/*	char	*service;
 /*	char	*queue_name;
 /*	char	*queue_id;
+/*	char	*encoding;
+/*	int	sendopts;
 /*	char	*sender;
 /*	char	*dsn_envid;
 /*	int	dsn_ret;
@@ -54,15 +58,12 @@
 #include <string.h>
 #include <ctype.h>
 
-#ifdef STRCASECMP_IN_STRINGS_H
-#include <strings.h>
-#endif
-
 /* Utility library. */
 
 #include <msg.h>
 #include <vstream.h>
 #include <name_mask.h>
+#include <stringops.h>
 
 /* Global library. */
 
@@ -74,6 +75,7 @@
 #include <verp_sender.h>
 #include <bounce.h>
 #include <dsn_mask.h>
+#include <rec_type.h>
 
 /* Application-specific. */
 
@@ -85,11 +87,11 @@
 
 int     bounce_notify_verp(int flags, char *service, char *queue_name,
 			           char *queue_id, char *encoding,
-			           char *recipient, char *dsn_envid,
-			           int dsn_ret, char *verp_delims,
-			           BOUNCE_TEMPLATES *ts)
+			           int sendopts, char *recipient,
+			           char *dsn_envid, int dsn_ret,
+			           char *verp_delims, BOUNCE_TEMPLATES *ts)
 {
-    char   *myname = "bounce_notify_verp";
+    const char *myname = "bounce_notify_verp";
     BOUNCE_INFO *bounce_info;
     int     bounce_status = 0;
     int     postmaster_status;
@@ -97,7 +99,8 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
     int     notify_mask = name_mask(VAR_NOTIFY_CLASSES, mail_error_masks,
 				    var_notify_classes);
     char   *postmaster;
-    VSTRING *verp_buf = vstring_alloc(100);
+    VSTRING *verp_buf;
+    VSTRING *new_id;
 
     /*
      * Sanity checks. We must be called only for undeliverable non-bounce
@@ -105,15 +108,46 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
      */
     if (*recipient == 0)
 	msg_panic("%s: attempt to bounce a single bounce", myname);
-    if (strcasecmp(recipient, mail_addr_double_bounce()) == 0)
+    if (strcasecmp_utf8(recipient, mail_addr_double_bounce()) == 0)
 	msg_panic("%s: attempt to bounce a double bounce", myname);
 
+    /*
+     * If the original sender requested REQUIRETLS, return headers only, and
+     * do not enforce REQUIRETLS for the delivery status notification.
+     */
+    if ((sendopts & SOPT_REQUIRETLS_ESMTP) != 0) {
+	dsn_ret = DSN_RET_HDRS;
+	sendopts &= ~SOPT_REQUIRETLS_ESMTP;
+    }
+
     /*
      * Initialize. Open queue file, bounce log, etc.
      */
     bounce_info = bounce_mail_init(service, queue_name, queue_id,
-				   encoding, dsn_envid, ts->failure);
+				   encoding, sendopts, dsn_envid,
+				   ts->failure);
 
+    /*
+     * If we have no recipient list then we can't send VERP replies. Send
+     * *something* anyway so that the mail is not lost in a black hole.
+     */
+    if (bounce_info->log_handle == 0) {
+	DSN_BUF *dsn_buf = dsb_create();
+	RCPT_BUF *rcpt_buf = rcpb_create();
+
+	dsb_simple(dsn_buf, "5.0.0", "(error report unavailable)");
+	(void) DSN_FROM_DSN_BUF(dsn_buf);
+	vstring_strcpy(rcpt_buf->address, "(recipient address unavailable)");
+	(void) RECIPIENT_FROM_RCPT_BUF(rcpt_buf);
+	bounce_status = bounce_one_service(flags, queue_name, queue_id,
+					   encoding, sendopts, recipient,
+					   dsn_envid, dsn_ret, rcpt_buf,
+					   dsn_buf, ts);
+	rcpb_free(rcpt_buf);
+	dsb_free(dsn_buf);
+	bounce_mail_free(bounce_info);
+	return (bounce_status);
+    }
 #define NULL_SENDER		MAIL_ADDR_EMPTY	/* special address */
 #define NULL_TRACE_FLAGS	0
 
@@ -121,20 +155,28 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
      * A non-bounce message was returned. Send a single bounce, one per
      * recipient.
      */
-    while (bounce_log_read(bounce_info->log_handle) != 0) {
+    verp_buf = vstring_alloc(100);
+    new_id = vstring_alloc(10);
+    while (bounce_log_read(bounce_info->log_handle, bounce_info->rcpt_buf,
+			   bounce_info->dsn_buf) != 0) {
+	RECIPIENT *rcpt = &bounce_info->rcpt_buf->rcpt;
 
 	/*
 	 * Notify the originator, subject to DSN NOTIFY restrictions.
+	 * 
+	 * Fix 20090114: Use the Postfix original recipient, because that is
+	 * what the VERP consumer expects.
 	 */
-	if (bounce_info->log_handle->rcpt.dsn_notify != 0	/* compat */
-	    && (bounce_info->log_handle->rcpt.dsn_notify & DSN_NOTIFY_FAILURE) == 0) {
+	if (rcpt->dsn_notify != 0		/* compat */
+	    && (rcpt->dsn_notify & DSN_NOTIFY_FAILURE) == 0) {
 	    bounce_status = 0;
 	} else {
-	    verp_sender(verp_buf, verp_delims, recipient,
-			bounce_info->log_handle->rcpt.address);
+	    verp_sender(verp_buf, verp_delims, recipient, rcpt);
 	    if ((bounce = post_mail_fopen_nowait(NULL_SENDER, STR(verp_buf),
-						 CLEANUP_FLAG_MASK_INTERNAL,
-						 NULL_TRACE_FLAGS)) != 0) {
+						 MAIL_SRC_MASK_BOUNCE,
+						 NULL_TRACE_FLAGS,
+						 sendopts,
+						 new_id)) != 0) {
 
 		/*
 		 * Send the bounce message header, some boilerplate text that
@@ -150,6 +192,9 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
 		    bounce_original(bounce, bounce_info, dsn_ret ?
 				    dsn_ret : DSN_RET_FULL);
 		bounce_status = post_mail_fclose(bounce);
+		if (bounce_status == 0)
+		    msg_info("%s: sender non-delivery notification: %s",
+			     queue_id, STR(new_id));
 	    } else
 		bounce_status = 1;
 
@@ -188,8 +233,10 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
 	    postmaster = var_bounce_rcpt;
 	    if ((bounce = post_mail_fopen_nowait(mail_addr_double_bounce(),
 						 postmaster,
-						 CLEANUP_FLAG_MASK_INTERNAL,
-						 NULL_TRACE_FLAGS)) != 0) {
+						 MAIL_SRC_MASK_BOUNCE,
+						 NULL_TRACE_FLAGS,
+						 sendopts,
+						 new_id)) != 0) {
 		if (bounce_header(bounce, bounce_info, postmaster,
 				  POSTMASTER_COPY) == 0
 		    && bounce_recipient_log(bounce, bounce_info) == 0
@@ -197,12 +244,15 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
 		    && bounce_recipient_dsn(bounce, bounce_info) == 0)
 		    bounce_original(bounce, bounce_info, DSN_RET_HDRS);
 		postmaster_status = post_mail_fclose(bounce);
+		if (postmaster_status == 0)
+		    msg_info("%s: postmaster non-delivery notification: %s",
+			     queue_id, STR(new_id));
 	    } else
 		postmaster_status = 1;
 
 	    if (postmaster_status)
-		msg_warn("postmaster notice failed while bouncing to %s",
-			 recipient);
+		msg_warn("%s: postmaster notice failed while bouncing to %s",
+			 queue_id, recipient);
 	}
     }
 
@@ -220,6 +270,7 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
      */
     bounce_mail_free(bounce_info);
     vstring_free(verp_buf);
+    vstring_free(new_id);
 
     return (bounce_status);
 }
